"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractExtendedContextFromHeader = extractExtendedContextFromHeader;
exports.withoutHeaders = withoutHeaders;
const zlib_1 = require("zlib");
/**
 * extractExtendedContextFromHeader 从 HTTP 请求头中提取上下文
 * @param headers
 * @param headerKey
 * @returns
 */
function extractExtendedContextFromHeader(headers, headerKey) {
    const empty = {};
    if (!headerKey) {
        return empty;
    }
    const headerValue = headers[headerKey.toLowerCase()];
    if (!headerValue) {
        return empty;
    }
    // Encode Base64 first, then gunzip
    try {
        const unzipped = (0, zlib_1.gunzipSync)(Buffer.from(headerValue, 'base64')).toString('utf-8');
        return JSON.parse(unzipped);
    }
    catch (e) {
        return empty;
    }
}
function withoutHeaders(headers, targetKeys) {
    const newHeaders = { ...headers };
    for (const target of targetKeys) {
        const th = target.toLowerCase();
        if (newHeaders[th]) {
            delete newHeaders[th];
        }
    }
    return newHeaders;
}
//# sourceMappingURL=invoker.js.map